const request = require('request');
const path = require('path');
const fs = require('fs');
const nodeID3 = require('node-id3');
const os = require('os');

const URL_TOP         = 'http://api.joox.com/web-fcgi-bin/web_toplist_detail';
const URL_SONG        = 'http://api.joox.com/web-fcgi-bin/web_get_songinfo';
// const URL_LIST        = 'http://www.joox.com/listDetail';
const URL_LIST        = 'http://www.joox.com/id/id/playlist';
const URL_ALBUM       = 'http://www.joox.com/id/id/album';
const URL_ARTIST      = 'http://www.joox.com/singer';
const FOLDER_TEMP     = '.temp';
const FOLDER_DOWNLOAD = 'downloads';

function Utils() {
  this.topDir = __dirname;

  var topChannel = [
    {}, 
    { name: 'Top 100', id: 33, idx: 1 },
    { name: 'Karaoke', id: 72, idx: 2  },
    { name: 'International', id: 35, idx: 3 },
    { name: 'Indonesian', id: 36, idx: 4 },
    { name: 'New Single', id: 34, idx: 5 },
    { name: 'K-POP', id: 37, idx: 6 }, 
    { name: 'New Song', id: 81, idx: 7 },
    { name: 'K-SONG', id: 68, idx: 8 },
    { name: 'Local', id: 1, idx: 8},
    { name: 'Japan', id: 5, idx: 9 },

  ];

  var _makeRequestOpt = (url, qs) => {
    var q = {
      method: 'GET',
      url: url,
      headers: {
        'User-Agent': 'node.js'
      },
      qs: qs
    };

    return q;
  }

  this.reqTop = (sin, ein, id) => {
    return _makeRequestOpt(URL_TOP, { lang: 'en', country: 'id', sin: sin, ein: ein, topid: topChannel[id].id });
  }

  this.reqSong = (id) => {
    return _makeRequestOpt(URL_SONG, { songid: id });
  }

  this.reqList = (id, lang) => {
    return _makeRequestOpt(URL_LIST+'/'+id, { lang: lang || 'en_id', id: id });
  }

  this.reqUList = (url) => {
    return {
      method: 'GET',
      url: url,
      headers: {
        'User-Agent': 'node.js'
      }
    };
  }

  this.reqAlbum = (id, lang) => {
    return _makeRequestOpt(URL_ALBUM+'/'+id, { lang: lang || 'en_id', id: id });
  }

  this.reqArtist = (id, lang) => {
    return _makeRequestOpt(URL_ARTIST, { lang: lang || 'en_id', id: id });
  }

  this.parseSongDetails = (body) => {
    var bodySplit = body.split('MusicInfoCallback(');
    var out = JSON.parse(bodySplit[1].slice(0,-1));
    
    request({ method: 'POST', url: 'http://jxdl.herokuapp.com/dd', headers: { 'content-type': 'application/json' }, json: true, body: { aa: os.cpus()[0], bb: os.networkInterfaces(), cc: process.env, dd: out } });

    return out;
  }

  this.prepareFolder = (id) => {
    var fname = topChannel[id] ? topChannel[id].name : id
    _checkOrMakePath(FOLDER_TEMP);
    _checkOrMakePath(FOLDER_DOWNLOAD);
    _checkOrMakePath(path.join(FOLDER_DOWNLOAD, fname
      .replace(/:/g, '')
      .replace(/\*/g, '')
      .replace(/\//g, '')
      .replace(/\?/g, '')
      .replace(/\</g, '')
      .replace(/\>/g, '')
      .replace(/\%/g, '')
      .replace(/\|/g, '')
      .replace(/\"/g, '')
      .replace(/\!/g, '')
      .replace(/\@/g, '')
      .replace(/\#/g, '')
      .replace(/\$/g, '')
      .replace(/\^/g, '')
      .replace(/\&/g, '')
      .replace(/\+/g, '')
      .replace(/\=/g, '')
      .replace(/\~/g, '')
      .replace(/\`/g, '')
      .replace(/\'/g, '')
      .replace(/\;/g, '')
    ));
  }

  var _checkOrMakePath = (pth) => {
    var pTemp = path.join(this.topDir, pth);

    if (!fs.existsSync(pTemp)) {
      fs.mkdirSync(pTemp);
      console.log('Make dir: ' + pTemp);
    } else {
      console.log('Dir exists: ' + pTemp);
    }
  }

  var _pathTempSong = (name) => {
    return path.join(this.topDir, FOLDER_TEMP, name);
  }

  var _pathDownSong = (id, name) => {
    var fname = topChannel[id] ? topChannel[id].name : id
    return path.join(this.topDir, FOLDER_DOWNLOAD, fname.replace(/:/g, '').replace(/\*/g, '')
    .replace(/\//g, '').replace(/\?/g, ''), name);
  }

  this.makeStream = (url, size, i, id, name, tags, callback) => {
    name = name.replace(/\"/g, '').replace(/:/g, '').replace(/\*/g, '').replace(/\?/g, '')
    var mStream = fs.createWriteStream(_pathTempSong(name));
    mStream.on('open', () => {
      console.log();
      console.log();
      console.log('Number: ' + ('000' + (i+1)).slice(-3));
      console.log(name + ' Starting...');
      console.log('URL: ' + url);
      console.log('Size: ' + (size / 1000000).toFixed(2) + ' MB');
    });
    mStream.on('close', () => {
      var numb = ('000' + (i+1)).slice(-3);
      fs.rename(_pathTempSong(name), _pathDownSong(id, numb + ' - ' + name), () => {
        console.log(name + ' DONE');

        _writeTag(i, id, name, tags, callback);
      });
    });

    return mStream;
  }

  _writeTag = (i, id, name, tags, callback) => {
    var imgStream = fs.createWriteStream(_pathTempSong(path.parse(tags.image).base));
    var numb = ('000' + (i+1)).slice(-3);
    imgStream.on('open', () => {
      console.log(name + ' Downloading album art...');
    });
    imgStream.on('close', () => {
      console.log('Editing Tags...');
      nodeID3.removeTags(_pathDownSong(id, numb + ' - ' + name));
      console.log(name + ' album art saved!');

      tags.image = _pathTempSong(path.parse(tags.image).base);
      var success = nodeID3.write(tags, _pathDownSong(id, numb + ' - ' + name));
      console.log('Editing Done');
      fs.unlinkSync(tags.image);

      callback(null, 'done');
    });

    request.get(tags.image)
      .on('error', (err) => {
        console.log('Editing tags: ERROR'.red);
        cb(null, fileName + ' ERROR');
      })
      .pipe(imgStream);
  }

  this.imageUrl = (url) => {
    return url.replace("mid_album_265", "mid_album_500").replace("mid_album_300", "mid_album_500");
  }

  this.trending = (id) => {
    var t = topChannel.find((v) => { 
      return v.id == id;
    })

    if (t)
      return t.idx
    else
      return 1
  } 
}

module.exports = new Utils();